/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.mendelu.savic.vida_cs_snake.model;

import cz.mendelu.savic.vida_cs_snake.controller.GameBoardController;
import java.io.IOException;
import java.util.ArrayList;
import javafx.scene.Group;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;

/**
 *
 * @author david
 */
public class Snake {

    private ArrayList<SnakePart> snakeParts;
    private final double width;
    private final Color color;
    private GameBoardController gameBoardController;
    private DirectionEnum direction;
    private boolean alive;
    private boolean grow;
    private boolean canChangeDirection;
    private final SnakePartPreLoader preloader;
    private Position newTailPosition = null;
    private boolean setDeadHead = true;

    public Snake(DirectionEnum startDirection, double x, double y, double width, Color color,
            GameBoardController gameBoardController) throws IOException {
        snakeParts = new ArrayList<>();
        preloader = new SnakePartPreLoader(width, color);
        this.width = width;
        this.color = color;
        this.gameBoardController = gameBoardController;
        this.direction = startDirection;
        this.alive = true;
        this.canChangeDirection = true;

        switch (startDirection) {
            case UP:
                snakeParts.add(preloader.getHeadUp(x, y));
                snakeParts.add(preloader.getBodyVertical(x, y + 1));
                snakeParts.add(preloader.getBodyVertical(x, y + 2));
                snakeParts.add(preloader.getEndDown(x, y + 3));
                break;
            case RIGHT:
                snakeParts.add(preloader.getHeadRight(x, y));
                snakeParts.add(preloader.getBodyHorizontal(x - 1, y));
                snakeParts.add(preloader.getBodyHorizontal(x - 2, y));
                snakeParts.add(preloader.getEndLeft(x - 3, y));
                break;
            case DOWN:
                snakeParts.add(preloader.getHeadDown(x, y));
                snakeParts.add(preloader.getBodyVertical(x, y - 1));
                snakeParts.add(preloader.getBodyVertical(x, y - 2));
                snakeParts.add(preloader.getEndUp(x, y - 3));
                break;
            case LEFT:
                snakeParts.add(preloader.getHeadLeft(x, y));
                snakeParts.add(preloader.getBodyHorizontal(x + 1, y));
                snakeParts.add(preloader.getBodyHorizontal(x + 2, y));
                snakeParts.add(preloader.getEndRight(x + 3, y));
                break;
        }
    }

    public ArrayList<SnakePart> getSnakeParts() {
        return snakeParts;
    }

    public ArrayList<Group> getSnakeElements() {
        ArrayList<Group> elements = new ArrayList<>();
        for (SnakePart snakePart : snakeParts) {
            elements.add(snakePart.getElement());
        }

        return elements;
    }

    public synchronized MoveValidity move(ArrayList<SnakePart> foodList, Pane root) throws IOException {
        if (!alive) {
            return MoveValidity.IS_ALREADY_DEAD;
        }

        SnakePart lastBodyPart = snakeParts.get(snakeParts.size() - 2);

        SnakePart oldHead = snakeParts.get(0);

        MoveValidity canMoveResult = gameBoardController.canMove(oldHead.getX(),
                oldHead.getY(), direction);
        
        if (canMoveResult != MoveValidity.VALID) {
            alive = false;
            return canMoveResult;
        }

        newTailPosition = new Position(lastBodyPart.getX(), lastBodyPart.getY());
        SnakePart newFirstBody = getNewFirstBody(direction);
        if (newFirstBody == null) {
            return MoveValidity.NOT_VALID;
        }
        Position newHeadPosition = getNewHeadPosition(direction);

        snakeParts.add(1, newFirstBody);

        grow = false;

        for (int i = 0; i < foodList.size(); i++) {
            if (foodList.get(i).getX() == newHeadPosition.getX() && foodList.get(i).getY() == newHeadPosition.getY()) {
                grow = true;
                foodList.remove(foodList.get(i));
                break;
            }
        }
        
        if (!grow) {
            preloader.removePart(lastBodyPart);
            snakeParts.remove(lastBodyPart);
        }

        canChangeDirection = true;

        return MoveValidity.VALID;
    }

    public void moveHeadAndTail() throws IOException {
        if (!alive) {
            if (setDeadHead) {
                setDeadHead();
            }
            return;
        }
        /**
         * HEAD part
         */
        SnakePart oldHead = snakeParts.get(0);

        SnakePart newHead = moveHead(direction);
        snakeParts.remove(oldHead);
        snakeParts.add(0, newHead);

        /**
         * TAIL part
         */
        if (!grow) {
            SnakePart oldTail = snakeParts.get(snakeParts.size() - 1);
            SnakePart newTail = moveTail();
            snakeParts.remove(oldTail);
            snakeParts.add(newTail);
        }
    }

    private void setDeadHead() {
        SnakePart oldHead = snakeParts.get(0);
        SnakePart newHead;

        SnakePartEnum deadPart = null;
        switch (oldHead.getPartEnum()) {
            case HEAD_UP:
                deadPart = SnakePartEnum.DEAD_HEAD_UP;
                break;
            case HEAD_RIGHT:
                deadPart = SnakePartEnum.DEAD_HEAD_RIGHT;
                break;
            case HEAD_DOWN:
                deadPart = SnakePartEnum.DEAD_HEAD_DOWN;
                break;
            case HEAD_LEFT:
                deadPart = SnakePartEnum.DEAD_HEAD_LEFT;
                break;
        }

        newHead = new SnakePart(deadPart, oldHead.getX(), oldHead.getY(), width,
                color);

        snakeParts.set(0, newHead);
        snakeParts.remove(oldHead);
        snakeParts.add(0, newHead);
        setDeadHead = false;
    }

    private SnakePart getNewFirstBody(DirectionEnum direction) throws IOException {
        SnakePart oldHead = snakeParts.get(0);
        SnakePart oldFirstBody = snakeParts.get(1);
        SnakePart newFirstBody = null;
        DirectionEnum secondDirection;

        double shiftX = oldHead.getX() - oldFirstBody.getX();
        double shiftY = oldHead.getY() - oldFirstBody.getY();

        if (shiftX == 0 && shiftY == 1) {
            secondDirection = DirectionEnum.UP;
        } else if (shiftX == 0 && shiftY == -1) {
            secondDirection = DirectionEnum.DOWN;
        } else if (shiftX == 1 && shiftY == 0) {
            secondDirection = DirectionEnum.LEFT;
        } else {
            secondDirection = DirectionEnum.RIGHT;
        }

        switch (direction) {
            case UP:
                switch (secondDirection) {
                    case UP:
                        return null;
                    case RIGHT:
                        newFirstBody = preloader.getCornerUpRight(oldHead.getX(), oldHead.getY());
                        break;
                    case DOWN:
                        newFirstBody = preloader.getBodyVertical(oldHead.getX(), oldHead.getY());
                        break;
                    case LEFT:
                        newFirstBody = preloader.getCornerUpLeft(oldHead.getX(), oldHead.getY());
                        break;
                }
                break;
            case RIGHT:
                switch (secondDirection) {
                    case UP:
                        newFirstBody = preloader.getCornerUpRight(oldHead.getX(), oldHead.getY());
                        break;
                    case RIGHT:
                        return null;
                    case DOWN:
                        newFirstBody = preloader.getCornerDownRight(oldHead.getX(), oldHead.getY());
                        break;
                    case LEFT:
                        newFirstBody = preloader.getBodyHorizontal(oldHead.getX(), oldHead.getY());
                        break;
                }
                break;
            case DOWN:
                switch (secondDirection) {
                    case UP:
                        newFirstBody = preloader.getBodyVertical(oldHead.getX(), oldHead.getY());
                        break;
                    case RIGHT:
                        newFirstBody = preloader.getCornerDownRight(oldHead.getX(), oldHead.getY());
                        break;
                    case DOWN:
                        return null;
                    case LEFT:
                        newFirstBody = preloader.getCornerDownLeft(oldHead.getX(), oldHead.getY());
                        break;
                }
                break;
            case LEFT:
                switch (secondDirection) {
                    case UP:
                        newFirstBody = preloader.getCornerUpLeft(oldHead.getX(), oldHead.getY());
                        break;
                    case RIGHT:
                        newFirstBody = preloader.getBodyHorizontal(oldHead.getX(), oldHead.getY());
                        break;
                    case DOWN:
                        newFirstBody = preloader.getCornerDownLeft(oldHead.getX(), oldHead.getY());
                        break;
                    case LEFT:
                        return null;

                }
                break;
        }

        return newFirstBody;
    }

    private SnakePart moveTail() throws IOException {
        SnakePart newLastBody = snakeParts.get(snakeParts.size() - 2);
        SnakePart newTail;

        double tailShiftX = newLastBody.getX() - newTailPosition.getX();
        double tailShiftY = newLastBody.getY() - newTailPosition.getY();

        if (tailShiftX == 0 && tailShiftY == 1) {
            newTail = preloader.getEndUp(newTailPosition.getX(), newTailPosition.getY());
        } else if (tailShiftX == 0 && tailShiftY == -1) {
            newTail = preloader.getEndDown(newTailPosition.getX(), newTailPosition.getY());
        } else if (tailShiftX == 1 && tailShiftY == 0) {
            newTail = preloader.getEndLeft(newTailPosition.getX(), newTailPosition.getY());
        } else {
            newTail = preloader.getEndRight(newTailPosition.getX(), newTailPosition.getY());
        }

        return newTail;
    }

    private SnakePart moveHead(DirectionEnum direction) throws IOException {
        SnakePart oldHead = snakeParts.get(0);
        SnakePart newHead;

        switch (direction) {
            case UP:
                newHead = preloader.getHeadUp(oldHead.getX(), oldHead.getY() - 1);
                break;
            case RIGHT:
                newHead = preloader.getHeadRight(oldHead.getX() + 1, oldHead.getY());
                break;
            case DOWN:
                newHead = preloader.getHeadDown(oldHead.getX(), oldHead.getY() + 1);
                break;
            case LEFT:
                newHead = preloader.getHeadLeft(oldHead.getX() - 1, oldHead.getY());
                break;
            default:
                newHead = null;
                break;
        }

        return newHead;
    }

    private Position getNewHeadPosition(DirectionEnum direction) throws IOException {
        SnakePart oldHead = snakeParts.get(0);

        switch (direction) {
            case UP:
                return new Position(oldHead.getX(), oldHead.getY() - 1);
            case RIGHT:
                return new Position(oldHead.getX() + 1, oldHead.getY());
            case DOWN:
                return new Position(oldHead.getX(), oldHead.getY() + 1);
            case LEFT:
                return new Position(oldHead.getX() - 1, oldHead.getY());
            default:
                return null;
        }
    }

    public void setDirection(DirectionEnum newDirection) {
        if (!canChangeDirection) {
            return;
        }

        if (newDirection == DirectionEnum.UP && this.direction == DirectionEnum.DOWN) {
            return;
        }
        if (newDirection == DirectionEnum.RIGHT && this.direction == DirectionEnum.LEFT) {
            return;
        }
        if (newDirection == DirectionEnum.DOWN && this.direction == DirectionEnum.UP) {
            return;
        }
        if (newDirection == DirectionEnum.LEFT && this.direction == DirectionEnum.RIGHT) {
            return;
        }
        this.direction = newDirection;
        this.canChangeDirection = false;
    }

    public DirectionEnum getDirection() {
        return direction;
    }

    public Color getColor() {
        return color;
    }
}
